/*
 * Copyright (c) 1996, 2003 VIA Networking Technologies, Inc.
 * All rights reserved.
 *
 * This software is copyrighted by and is the sole property of * VIA Networking Technologies, Inc. This software may only be used
 * in accordance with the corresponding license agreement. Any unauthorized
 * use, duplication, transmission, distribution, or disclosure of this
 * software is expressly forbidden.
 *
 * This software is provided by VIA Networking Technologies, Inc. "as is"
 * and any express or implied warranties, including, but not limited to, the
 * implied warranties of merchantability and fitness for a particular purpose
 * are disclaimed. In no event shall VIA Networking Technologies, Inc.
 * be liable for any direct, indirect, incidental, special, exemplary, or
 * consequential damages.
 *
 *
 * File: osdep.h
 *
 * Purpose: OS depended function and macro defined, including register accessing.
 *
 * Author: Guard Kuo
 *
 * Date: Jan 28, 2005
 *
 *
 */
#ifndef ___OSDEP_H__
#define ___OSDEP_H__

#include <sys/types.h>
#include <sys/param.h>
#include <sys/systm.h>
#include <sys/mbuf.h>
#include <sys/protosw.h>
#include <sys/socket.h>
#include <sys/malloc.h>
#include <sys/kernel.h>
#include <machine/bus_pio.h>
#include <machine/bus_memio.h>
#include <machine/bus.h>
#include <vm/vm.h>
#include <vm/pmap.h>
#include <machine/clock.h>

#if __FreeBSD_version >= 400000
#include <machine/resource.h>
#include <sys/bus.h>
#include <sys/rman.h>
#endif /*__FreeBSD_version*/

#if __FreeBSD_version >= 502000
#include <dev/pci/pcireg.h>
#include <dev/pci/pcivar.h>
#else
#include <pci/pcireg.h>
#include <pci/pcivar.h>
#endif

typedef u_int8_t    UCHAR,  *PUCHAR;
typedef u_int16_t   U16,    *PU16;
typedef u_int8_t    UINT8,  *PUINT8;
typedef u_int32_t   U32,    *PU32;
typedef u_int32_t   UINT32, *PUINT32;
typedef u_int32_t   UINT,   *PUINT;
typedef u_int8_t    BYTE,   *PBYTE;
typedef u_int8_t    U8,     *PU8;
typedef u_int32_t   BOOL,   *PBOOL;
typedef u_int16_t   WORD,   *PWORD;
typedef u_int32_t   DWORD,  *PDWORD;
typedef unsigned long   ULONG,  *PULONG;

#ifndef dma_addr_t
#define dma_addr_t	U32
#endif

#ifndef FALSE
#define FALSE   (0)
#endif

#ifndef TRUE
#define TRUE    (!(FALSE))
#endif
/*
   little/big endian converting
*/
#ifndef cpu_to_le32
#define cpu_to_le32(x) x
#endif

#ifndef cpu_to_le16
#define cpu_to_le16(x) x
#endif


/*
 * Message Output function
 */
#if __FreeBSD_version < 500000
#define VELOCITY_HW_PRT(hw, l, p, args...) do { printf("getb%d: " p, hw->back->unit); } while(0)
#define VELOCITY_PRT(msglevel, l, p, args...) do { printf( p); } while(0)
#else
#define VELOCITY_HW_PRT(hw, l, p, args...) do { printf("getb%d: " p, hw->back->unit, ##args); } while(0)
#define VELOCITY_PRT(msglevel, l, p, args...) do { printf( p, ##args); } while(0)
#endif
/*
   delay function
*/
#ifndef mdelay
#define mdelay(x)   DELAY(1000*(x))
#endif

#ifndef udelay
#define udelay(x)   DELAY((x))
#endif


struct em_osdep
{
    bus_space_tag_t    mem_bus_space_tag;
    bus_space_handle_t mem_bus_space_handle;
    bus_space_tag_t    IO_bus_btag;         /* bus space tag for IO*/
    bus_space_handle_t IO_bus_bhandle;      /* bus space handle for IO*/
#if __FreeBSD_version >= 400000
    struct device     *dev;
#endif
    int unit;
};


/*
 * register space access macros
 */
#define CSR_WRITE_4(hw, val, reg) \
        bus_space_write_4( ((struct em_osdep*)(hw)->back)->mem_bus_space_tag, \
                           ((struct em_osdep*)(hw)->back)->mem_bus_space_handle, reg, val)

#define CSR_WRITE_2(hw, val, reg) \
        bus_space_write_2( ((struct em_osdep*)(hw)->back)->mem_bus_space_tag, \
                           ((struct em_osdep*)(hw)->back)->mem_bus_space_handle, reg, val)

#define CSR_WRITE_1(hw, val, reg) \
        bus_space_write_1( ((struct em_osdep*)(hw)->back)->mem_bus_space_tag, \
                           ((struct em_osdep*)(hw)->back)->mem_bus_space_handle, reg, val)

#define CSR_READ_4(hw, reg) \
        bus_space_read_4( ((struct em_osdep*)(hw)->back)->mem_bus_space_tag, \
                          ((struct em_osdep*)(hw)->back)->mem_bus_space_handle, reg)

#define CSR_READ_2(hw, reg) \
        bus_space_read_2( ((struct em_osdep*)(hw)->back)->mem_bus_space_tag, \
                          ((struct em_osdep*)(hw)->back)->mem_bus_space_handle, reg)
#define CSR_READ_1(hw, reg) \
        bus_space_read_1( ((struct em_osdep*)(hw)->back)->mem_bus_space_tag, \
                          ((struct em_osdep*)(hw)->back)->mem_bus_space_handle, reg)


#define _INB(hw, reg) \
        bus_space_read_1(((struct em_osdep*)(hw)->back)->IO_bus_btag, \
                         ((struct em_osdep*)(hw)->back)->IO_bus_bhandle, reg)
                                       
#define _OUTB(hw, val, reg) \
        bus_space_write_1( ((struct em_osdep*)(hw)->back)->IO_bus_btag, \
                           ((struct em_osdep*)(hw)->back)->IO_bus_bhandle, reg, val)


/* PCI configuration register read/write interface */
#if __FreeBSD_version < 400000
#define PCI_READ_1(dev, reg)        pci_conf_read(dev, reg) & 0x000000FF
#define PCI_READ_2(dev, reg)        pci_conf_read(dev, reg) & 0x0000FFFF
#define PCI_READ_4(dev, reg)        pci_conf_read(dev, reg)
#define PCI_WRITE_1(dev, reg, val)  pci_conf_write(dev, reg, ((pci_read_conf(dev, reg) & 0xffffff00) | val) )
#define PCI_WRITE_2(dev, reg, val)  pci_conf_write(dev, reg, ((pci_read_conf(dev, reg) & 0xffff0000) | val) )
#define PCI_WRITE_4(dev, reg, val)  pci_conf_write(dev, reg, val )
#else
#define PCI_READ_1(dev, reg)        pci_read_config(dev, reg, 1)
#define PCI_READ_2(dev, reg)        pci_read_config(dev, reg, 2)
#define PCI_READ_4(dev, reg)        pci_read_config(dev, reg, 4)
#define PCI_WRITE_1(dev, reg, val)  pci_write_config(dev, reg, val, 1)
#define PCI_WRITE_2(dev, reg, val)  pci_write_config(dev, reg, val, 2)
#define PCI_WRITE_4(dev, reg, val)  pci_write_config(dev, reg, val, 4)
#endif

#ifndef wmb
#define wmb()
#endif
/* some OS doesn't support priority transmit, so we can set the number of queue to 1 */

#define TX_QUEUE_NO         1
#define TX_DEFAULT_QUEUE    0

#ifndef ETH_FRAME_LEN
#define ETH_FRAME_LEN 1514
#endif

/* status update */
#define rhine_reset_rx_stats(pStats) \
    (pStats)->rx_errors=0;    \
    (pStats)->rx_packets=0


#define rhine_reset_tx_stats(pStats)   \
    (pStats)->tx_packets=0;   \
    (pStats)->tx_errors=0;    \
    (pStats)->collisions=0


#define rhine_update_tx_stats(dst, src) \
    (dst)->if_opackets          +=(src).tx_packets; \
    (dst)->if_oerrors           +=(src).tx_errors; \
    (dst)->if_collisions        +=(src).collisions

#define rhine_update_rx_stats(dst, src) \
    (dst)->if_ierrors       += (src).rx_errors; \
    (dst)->if_ipackets      += (src).rx_packets



/* memory allocator */
#if __FreeBSD_version >= 400000
#define alloc_conti_mem(addr, size, addr_dma) \
    addr = contigmalloc( size, M_DEVBUF, M_NOWAIT, 0, 0xffffffff, PAGE_SIZE, 0); \
    addr_dma = vtophys(addr)
#else
#define alloc_conti_mem(addr, size, addr_dma) \
    addr = malloc(size, M_DEVBUF, M_NOWAIT); \
    addr_dma = vtophys(addr)
#endif

#define alloc_mem(addr, size) \
    addr = malloc(size, M_DEVBUF, M_NOWAIT)


#ifndef ETH_ZLEN
#define ETH_ZLEN 60
#endif

typedef struct _rhine_rd_info {
    struct mbuf*        mbuf;
    dma_addr_t          mbuf_dma;
} GETB_RD_INFO, *PGETB_RD_INFO;

typedef struct _rhine_td_info {
    struct mbuf*        mbuf;
} GETB_TD_INFO, *PGETB_TD_INFO;


#endif

